@@ -26,7 +26,6 @@
 #include <time.h>
 
 #include <glib.h>
-#include <gnome.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -86,19 +85,35 @@ list__process_line (char     *line,
 	if (line[0] == 'D') {
 		if (comm_iso->cur_path != NULL)
 			g_free (comm_iso->cur_path);
-		comm_iso->cur_path = g_strdup (get_last_field(line, 4));
+		comm_iso->cur_path = g_strdup (get_last_field (line, 4));
 		
-	} else if (line[0] == '-') {
-		/* Is file */
+	} else if (line[0] == '-') { /* Is file */
+		const char *last_field, *first_bracket;
+
 		fdata = file_data_new ();
 		
 		fields = split_line (line, 8);
-		fdata->size = atol (fields[4]);
+		fdata->size = atoll (fields[4]);
 		fdata->modified = mktime_from_string (fields[5], fields[6], fields[7]);
 		g_strfreev (fields);
 		
 		/* Full path */
-		name_field = get_last_field (line, 12);
+
+		last_field = get_last_field (line, 9);
+		first_bracket = strchr (last_field, ']');
+		if (first_bracket == NULL) {
+			file_data_free (fdata);
+			return;
+		}
+			
+		name_field = eat_spaces (first_bracket + 1);
+		if ((name_field == NULL) 
+		    || (strcmp (name_field, ".") == 0)
+		    || (strcmp (name_field, "..") == 0)) {
+			file_data_free (fdata);
+			return;
+		}
+
 		fdata->original_path = fdata->full_path = g_strstrip (g_strconcat (comm_iso->cur_path, name_field, NULL));
 		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 		fdata->path = remove_level_from_path (fdata->full_path);
@@ -116,7 +131,8 @@ fr_command_iso_list (FRCommand *comm)
 				      comm);
 
 	fr_process_begin_command (comm->process, "isoinfo");
-	fr_process_add_arg (comm->process, "-R -J -l -i");
+	fr_process_add_arg (comm->process, "-J -R");
+	fr_process_add_arg (comm->process, "-l -i");
 	fr_process_add_arg (comm->process, comm->e_filename);
 	fr_process_end_command (comm->process);
 	fr_process_start (comm->process);
@@ -132,37 +148,60 @@ fr_command_iso_extract (FRCommand  *comm,
 			gboolean    junk_paths,
 			const char *password)
 {
+	char  *e_dest_dir = fr_command_escape (comm, dest_dir);
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char       *scanpath = (char*)scan->data;
+		char       *path = scan->data;
 		const char *filename;
-                char       *fullpath, *temppath = NULL;
+                char       *file_dir, *e_temp_dest_dir = NULL, *temp_dest_dir = NULL;
 	
-		filename = file_name_from_path (scanpath);
-		temppath = remove_level_from_path (scanpath+1); /* Add one to ignore initial '/' */
-		if (temppath == NULL)
-			temppath = g_strdup ("/");
-		if (dest_dir != NULL) {
-			fullpath = g_build_filename (dest_dir, temppath, NULL);
-			g_free (temppath);
-		} else 
-			fullpath = temppath;
+		filename = file_name_from_path (path);
+		file_dir = remove_level_from_path (path);
+		if ((file_dir != NULL) && (strcmp (file_dir, "/") != 0)) 
+			e_temp_dest_dir = g_build_filename (e_dest_dir, file_dir, NULL);
+		 else 
+			e_temp_dest_dir = g_strdup (e_dest_dir);
+		g_free (file_dir);
+		if (e_temp_dest_dir == NULL) 
+			continue;
 
-		ensure_dir_exists (fullpath, 0700);
+		temp_dest_dir = unescape_str (e_temp_dest_dir);
+		ensure_dir_exists (temp_dest_dir, 0700);
 
 		fr_process_begin_command (comm->process, "isoinfo");
-		fr_process_set_working_dir (comm->process, fullpath);
-		fr_process_add_arg (comm->process, "-R -J -i");
+		fr_process_set_working_dir (comm->process, temp_dest_dir);
+		fr_process_add_arg (comm->process, "-J -R");
+		fr_process_add_arg (comm->process, "-i");
 		fr_process_add_arg (comm->process, comm->e_filename);
 		fr_process_add_arg (comm->process, "-x");
-		fr_process_add_arg (comm->process, scanpath);
+		fr_process_add_arg (comm->process, path);
 		fr_process_add_arg (comm->process, ">");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
-		fr_process_start (comm->process);
-		
-		g_free (fullpath);
+	
+		g_free (e_temp_dest_dir);
+		g_free (temp_dest_dir);
+	}
+
+	g_free (e_dest_dir);
+
+	fr_process_start (comm->process);
+}
+
+
+static void
+fr_command_iso_handle_error (FRCommand   *comm, 
+			     FRProcError *error)
+{
+	if (error->type == 2) { /* ERROR: Unable to find Joliet SVD */
+
+		/* Remove the -J -R options and start again */
+		fr_process_set_arg_at (comm->process, 
+				       comm->process->error_command, 
+				       1, 
+				       "");
+		comm->process->restart = TRUE;
 	}
 }
 
@@ -180,6 +219,7 @@ fr_command_iso_class_init (FRCommandIsoClass *class)
 
         afc->list           = fr_command_iso_list;
 	afc->extract        = fr_command_iso_extract;
+	afc->handle_error   = fr_command_iso_handle_error;
 }
 
 
@@ -248,6 +288,10 @@ fr_command_iso_new (FRProcess  *process,
 {
 	FRCommand *comm;
 
+	if (!is_program_in_path("isoinfo")) {
+		return NULL;
+	}
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_ISO, NULL));
 	fr_command_construct (comm, process, filename);
 
